/* 
 * Compared to the other of Tracking Exposed,
 * this script has been upgraded to use mongo4 */

const _ = require('lodash');
const MongoClient = require('mongodb').MongoClient;
const debug = require('debug')('lib:mongo');
const nconf = require('nconf');


var savedMongoUri = null;
function mongoUri(forced) {

    // by passing 'null' you'll reset mongoUri
    if(_.isNull(forced))
        savedMongoUri = null;

    if(forced && forced.uri)
        savedMongoUri = forced.uri;

    if(savedMongoUri)
        return savedMongoUri;

    /* if is not yet set, reset using the config */
    const mongoHost = nconf.get('mongoHost');
    const mongoPort = nconf.get('mongoPort');
    const mongoDb = nconf.get('mongoDb');

    if(!mongoHost || !mongoPort || !mongoDb)
        throw new Error("configuration missing");

    savedMongoUri = `mongodb://${mongoHost}:${mongoPort}/${mongoDb}`;
    debug("Initializing mongoUri as %s", savedMongoUri);
    return savedMongoUri;
}

async function clientConnect(config) {
    /* concurrency: <Int>, uri: <mongo address> */
    if(!config) config = {};

    const uri = mongoUri();

    try {
        const client = new MongoClient(uri);
        await client.connect();
        return client;
    } catch(err) {
        debug("Error in connecting to mongo [%s]: %s", uri, err.message);
        return null;
    }
}

async function listCollections(mongoc) {
    return mongoc
        .db()
        .listCollections()
        .toArray();
};

async function writeOne(mongoc, cName, doc) {
    debug("write in [%s] with %O", cName, _.keys(doc));
    return mongoc
        .db()
        .collection(cName)
        .insertOne(doc);
};

async function insertMany(mongoc, cName, docs, options) {
    if(!options) options = {};
    return mongoc
        .db()
        .collection(cName)
        .insertMany(docs, options ? options : {});
};

async function updateOne(mongoc, cName, selector, updated) {
    return mongoc
        .db()
        .collection(cName)
        .updateOne(selector, { $set: updated });
};

async function upsertOne(mongoc, cName, selector, updated) {
    return mongoc
        .db()
        .collection(cName)
        .updateOne(selector, { $set: updated }, { upsert: true});
};

async function read(mongoc, cName, selector, sorter) {
    return mongoc
        .db()
        .collection(cName)
        .find(selector)
        .sort(sorter ? sorter : {})
        .toArray();
};

async function readOne(mongoc, cName, selector, sorter) {
    const l = await read(mongoc, cName, selector, sorter);
    if(_.size(l) > 1)
        debug("Warning, readOne %j returned %d docs", selector, _.size(l));
    return _.first(l);
};

async function deleteMany(mongoc, cName, selector) {
    if(_.size(_.keys(selector)) === 0)
        throw new Error("Not in my watch: you can't delete everything with this library");
    return mongoc
        .db()
        .collection(cName)
        .deleteMany(selector);
};

async function readLimit(mongoc, cName, selector, sorter, limitN, past) {
    if(!limitN)
        throw new Error("Not specified the amount of documents expected");
    return mongoc
        .db()
        .collection(cName)
        .find(selector)
        .sort(sorter)
        .skip(past ? past : 0)
        .limit(limitN)
        .toArray();
};

async function count(mongoc, cName, selector) {
    return mongoc
        .db()
        .collection(cName)
        .countDocuments(selector);
};


async function createIndex(mongoc, cName, index, opt) {
    return mongoc
        .db()
        .createIndex(cName, index, opt);
};

async function distinct(mongoc, cName, field, query) {
    return mongoc
        .db()
        .collection(cName)
        .distinct(field, query);
};

async function aggregate(mongoc, cName, pipeline) {
    return mongoc
        .db()
        .collection(cName)
        .aggregate(pipeline)
        .toArray();
};


async function checkMongoWorks() {
    try {
        const mongoc = await clientConnect({concurrency: 1});
        const results = await listCollections(mongoc);
        await mongoc.close();
        return results;
    } catch(error) {
        debug("Failure in checkMongoWorks: %s", error.message);
        throw error;
    }
};

module.exports = {
    clientConnect,
    mongoUri,

    listCollections,
    writeOne,
    insertMany,
    updateOne,
    upsertOne,
    readOne,
    read,
    readLimit,
    deleteMany,
    count,
    createIndex,
    distinct,
    aggregate,

    checkMongoWorks
};
